@@ -19,7 +19,7 @@ from ._pre_data import _pre_data
 from ._common import _design_matrix, _leg_mt, _full_col_rank
 
 
-def unbalance_longwas_fixed(data_file, id, tpoint, trait, bed_file, agmat_file, var_com, snp_lst=None,
+def unbalance_longwas_fixed_(data_file, id, tpoint, trait, bed_file, agmat_file, var_com, condition_snp=None, snp_lst=None,
             tfix=None, fix=None, forder=3, aorder=3, porder=3, na_method='omit',
                              prefix_outfile='unbalance_longwas_fixed'):
     """
@@ -34,6 +34,7 @@ def unbalance_longwas_fixed(data_file, id, tpoint, trait, bed_file, agmat_file,
     :param agmat_file: the file for genomic relationship matrix. This file can be produced by
     gmat.gmatrix.agmat function using agmat(bed_file, inv=True, small_val=0.001, out_fmt='id_id_val')
     :param var_com: the estimated variance parameters by the gmat.longwas.unbalance.unbalance_varcom function.
+    :param condition_snp: SNP ID or a list of SNP ID for conditional snps added to the time fixed effect.
     :param snp_lst: the snp list to test. Default is None.
     :param tfix: A class variate name for the time varied fixed effect. Default value is None. Only one time varied
     fixed effect can be included in the current version.
@@ -75,6 +76,30 @@ def unbalance_longwas_fixed(data_file, id, tpoint, trait, bed_file, agmat_file,
         xmat = np.concatenate([xmat_t, xmat_nt[:, 1:]], axis=1)
     if np.linalg.matrix_rank(xmat) < xmat.shape[1]:
         xmat = _full_col_rank(xmat)
+    logging.info("Conditional SNP: {}".format(condition_snp))
+    snp_on_disk = Bed(bed_file, count_A1=False)
+    fam_df = pd.read_csv(bed_file + '.fam', sep='\s+', header=None)
+    id_geno = list(np.array(fam_df.iloc[:, 1], dtype=str))
+    id_order_index = []
+    for i in id_order:
+        id_order_index.append(id_geno.index(i))
+    leg_lst = []  # legendre polynomials for time dependent fixed SNP effects, save for each individuals
+    for val in id_order:
+        leg_val = _leg_mt(data_df[data_df[id] == val][tpoint], tmax, tmin, forder)
+        leg_val = np.concatenate(leg_val, axis=1)
+        leg_lst.append(leg_val)
+    if condition_snp is not None:
+        if not isinstance(condition_snp, list):
+            condition_snp = [condition_snp]
+        condition_snp_index = snp_on_disk.sid_to_index(condition_snp)
+        condition_snp_val = snp_on_disk[id_order_index, condition_snp_index].read().val
+        if np.any(np.isnan(condition_snp_val)):
+            logging.info('Missing genotypes of conditional SNP are imputed with random genotypes.')
+            condition_snp_val = impute_geno(condition_snp_val)
+        for i in range(condition_snp_val.shape[1]):
+            snp_condition = list(map(lambda x, y: x * y, leg_lst, list(condition_snp_val[:, i])))
+            snp_condition = np.concatenate(snp_condition, axis=0)
+            xmat = np.concatenate((xmat, snp_condition), axis=1)
     logging.info('***Build the dedign matrix for random effect***')
     logging.info('Legendre order for additive effects: {}'.format(aorder))
     leg_add = _leg_mt(data_df[tpoint], tmax, tmin, aorder)
@@ -130,15 +155,9 @@ def unbalance_longwas_fixed(data_file, id, tpoint, trait, bed_file, agmat_file,
     xvx = linalg.inv(xvx)
     pmat = vmat - reduce(np.dot, [xv.T, xvx, xv])
     logging.info('***Read the snp data***')
-    snp_on_disk = Bed(bed_file, count_A1=False)
     num_id = snp_on_disk.iid_count
     num_snp = snp_on_disk.sid_count
     logging.info("There are {:d} individuals and {:d} SNPs.".format(num_id, num_snp))
-    fam_df = pd.read_csv(bed_file + '.fam', sep='\s+', header=None)
-    id_geno = list(np.array(fam_df.iloc[:, 1], dtype=str))
-    id_order_index = []
-    for i in id_order:
-        id_order_index.append(id_geno.index(i))
     if snp_lst is None:
         snp_lst = range(num_snp)
     snp_lst = list(snp_lst)
@@ -152,19 +171,17 @@ def unbalance_longwas_fixed(data_file, id, tpoint, trait, bed_file, agmat_file,
     logging.info('#####################################################################')
     logging.info('###Start the fixed regression longitudinal GWAS for unbalance data###')
     logging.info('#####################################################################')
-    leg_lst = []  # legendre polynomials for time dependent fixed SNP effects, save for each individuals
-    for val in id_order:
-        leg_val = _leg_mt(data_df[data_df[id] == val][tpoint], tmax, tmin, forder)
-        leg_val = np.concatenate(leg_val, axis=1)
-        leg_lst.append(leg_val)
-    # leg_mat = block_diag(leg_lst, format='csr')
     tpoint_vec = sorted(set(data_df[tpoint]))
     leg_tpoint_mat = _leg_mt(np.array(tpoint_vec), tmax, tmin, forder)
     leg_tpoint_mat = np.concatenate(leg_tpoint_mat, axis=1)
     leg_tpoint_accum = np.sum(leg_tpoint_mat, axis=0)
     chi_df = leg_lst[0].shape[1]
+    eff_main_vec = []
+    se_main_vec = []
+    p_main_vec = []
     eff_vec = []
     chi_vec = []
+    p_gbye_vec = []
     p_vec = []
     p_min_vec = []
     eff_accum_vec = []
@@ -177,11 +194,26 @@ def unbalance_longwas_fixed(data_file, id, tpoint, trait, bed_file, agmat_file,
         snp_fix = np.concatenate(snp_fix, axis=0)
         # snp_fix = leg_mat.dot((sparse.kron(snp_mat[:, i:(i+1)], eye_forder)))
         # snp_fix = snp_fix.toarray()
+        p_xsnp = np.dot(pmat, snp_fix[:, 0:1])
+        xpx = np.dot(snp_fix[:, 0:1].T, p_xsnp)
+        se_main = np.sqrt(xpx[0, 0])
+        xpy = np.dot(p_xsnp.T, y)
+        eff_main = xpy[-1, -1]
+        chi_val = eff_main*eff_main/(se_main*se_main)
+        p_main = chi2.sf(chi_val, 1)
+        eff_main_vec.append(eff_main)
+        se_main_vec.append(se_main)
+        p_main_vec.append(p_main)
         p_xsnp = np.dot(pmat, snp_fix)
         xpx = np.dot(snp_fix.T, p_xsnp)
         xpx = linalg.inv(xpx)
         xpy = np.dot(p_xsnp.T, y)
         b = np.dot(xpx, xpy)
+        eff = b[-(chi_df-1):, -1]
+        eff_var = xpx[-(chi_df-1):, -(chi_df-1):]
+        chi_val = np.sum(np.dot(np.dot(eff.T, linalg.inv(eff_var)), eff))
+        p_gbye = chi2.sf(chi_val, chi_df-1)
+        p_gbye_vec.append(p_gbye)
         eff = b[-chi_df:, -1]
         eff_var = xpx[-chi_df:, -chi_df:]
         chi_val = np.sum(np.dot(np.dot(eff.T, linalg.inv(eff_var)), eff))
@@ -224,12 +256,16 @@ def unbalance_longwas_fixed(data_file, id, tpoint, trait, bed_file, agmat_file,
     res_df.columns = ['chro', 'snp_ID', 'pos', 'allele1', 'allele2']
     res_df.loc[:, 'order'] = snp_lst
     res_df = res_df.iloc[:, [5, 0, 1, 2, 3, 4]]
+    res_df.loc[:, 'eff_main'] = eff_main_vec
+    res_df.loc[:, 'se_main'] = se_main_vec
+    res_df.loc[:, 'p_main'] = p_main_vec
     eff_vec = np.array(eff_vec)
     for i in range(eff_vec.shape[1]):
         col_ind = 'eff' + str(i)
         res_df.loc[:, col_ind] = eff_vec[:, i]
     res_df.loc[:, 'eff_accum'] = eff_accum_vec
     res_df.loc[:, 'chi_val'] = chi_vec
+    res_df.loc[:, 'p_gbye'] = p_gbye_vec
     res_df.loc[:, 'p_val'] = p_vec
     res_df.loc[:, 'p_min'] = p_min_vec
     res_df.loc[:, 'p_accum'] = p_accum_vec
